﻿// Copyright (c) 2008, NTT DATA Corporation.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using TERASOLUNA.Fw.Client.Configuration.Conversion;
using TERASOLUNA.Fw.Utlib.Logging;
using TERASOLUNA.Fw.Common.Logging;
namespace TERASOLUNA.Fw.ClientUT.Configuration.Conversion
{
    /// <summary>
    ///TERASOLUNA.Fw.Client.Configuration.Conversion.ResultElement のテスト クラスです。すべての
    ///TERASOLUNA.Fw.Client.Configuration.Conversion.ResultElement 単体テストをここに含めます
    ///</summary>
    [TestClass()]
    public class ResultElementTest
    {
        private TestContext testContextInstance;

        ///<summary>
        /// 現在のテストの実行についての情報および機能を
        /// 提供するテスト コンテキストを取得または設定します。
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }
        #region 追加のテスト属性
        // 
        //テストを作成するときに、次の追加属性を使用することができます:
        //
        //クラスの最初のテストを実行する前にコードを実行するには、ClassInitialize を使用
        //
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //クラスのすべてのテストを実行した後にコードを実行するには、ClassCleanup を使用
        //
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //各テストを実行する前にコードを実行するには、TestInitialize を使用
        //
        [TestInitialize()]
        public void MyTestInitialize()
        {
            LogUTUtil.Clear();
        }
        //
        //各テストを実行した後にコードを実行するには、TestCleanup を使用
        //
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion


        ///<summary>
        /// ClearViews のテスト
        ///</summary>
        ///<remarks>
        /// コンストラクタでClearViewCollectionを設定し、その値を取得した場合、
        /// 一致することを確認するテスト。
        ///</remarks>
        [TestMethod()]
        public void ClearViewsTest01()
        {
            //準備
            ClearViewCollection expectedClearViews = new ClearViewCollection();
            ColumnCollection expectedColumns = new ColumnCollection();
            ResultElement target = new ResultElement(expectedClearViews, expectedColumns);

            //実行
            ClearViewCollection actual = target.ClearViews;

            //検証
            Assert.AreSame(expectedClearViews, actual);

            // ログの確認
            Assert.AreEqual(0, LogUTUtil.CountLogs(typeof(ResultElement), LogLevel.INFO));

        }

        ///<summary>
        /// Columns のテスト
        ///</summary>
        ///<remarks>
        /// コンストラクタでColumnCollectionを設定し、その値を取得した場合、
        /// 一致することを確認するテスト。
        ///</remarks>
        [TestMethod()]
        public void ColumnsTest01()
        {
            //準備
            ClearViewCollection expectedClearViews = new ClearViewCollection();
            ColumnCollection expectedColumns = new ColumnCollection();
            ResultElement target = new ResultElement(expectedClearViews, expectedColumns);

            //実行
            ColumnCollection actual = target.Columns;

            //検証
            Assert.AreSame(expectedColumns, actual);

            // ログの確認
            Assert.AreEqual(0, LogUTUtil.CountLogs(typeof(ResultElement), LogLevel.INFO));

        }

        ///<summary>
        /// ResultElement () のテスト
        ///</summary>
        ///<remarks>
        /// 引数無しのコンストラクタを呼び出した場合、インスタンスが返却されることを確認するテスト。
        ///</remarks>
        [TestMethod()]
        public void ConstructorTest01()
        {
            //実行
            ResultElement target = new ResultElement();

            //検証
            Assert.IsNotNull(target);

            // ログの確認
            Assert.AreEqual(0, LogUTUtil.CountLogs(typeof(ResultElement), LogLevel.INFO));

        }

        ///<summary>
        /// ResultElement (ClearViewCollection, ColumnCollection) のテスト
        ///</summary>
        ///<remarks>
        /// ClearViewCollection、ColumnCollectionを引数とするコンストラクタを呼び出した場合、
        /// 引数が正常に設定されていることを確認するテスト。
        ///</remarks>
        [TestMethod()]
        public void ConstructorClearViewCollectionColumnCollectionTest01()
        {
            //準備
            ClearViewCollection expectedClearViews = new ClearViewCollection();
            ColumnCollection expectedColumns = new ColumnCollection();

            //実行
            ResultElement target = new ResultElement(expectedClearViews, expectedColumns);

            //検証
            Assert.AreSame(expectedClearViews, target.ClearViews);
            Assert.AreSame(expectedColumns, target.Columns);

            // ログの確認
            Assert.AreEqual(0, LogUTUtil.CountLogs(typeof(ResultElement), LogLevel.INFO));

        }

    }

}
